// SfsProvider
#include "SfsProvider.h"

// SfsWin
#include "SfsWin.h"

// STL
#include <map>
#include <memory>


using namespace std;


class SfsWins
{
public:
	static bool HandleInit(unsigned int id, SFS_Init const* init, SFS_Context* context)
	{
		auto_ptr<SfsWin> sfsWin(new SfsWin);
		if (!s_sfsWins.insert(SfsWinMap::value_type(id, sfsWin.get())).second)
		{
			context->SetException(context, L"SfsWins::HandleInit(): Invalid provider ID");
			return false;
		}
		return sfsWin.release()->HandleInit(init, context);
	}

	static void HandleRequest(unsigned int id, SFS_Request const* request, SFS_Context* context)
	{
		SfsWinMap::iterator it = s_sfsWins.find(id);
		if (it == s_sfsWins.end())
			context->SetException(context, L"SfsWins::HandleRequest(): Invalid provider ID");
		else
			return it->second->HandleRequest(request, context);
	}

	static void HandleDestroy(unsigned int id, SFS_Context* context)
	{
		SfsWinMap::iterator it = s_sfsWins.find(id);
		if (it == s_sfsWins.end())
			context->SetException(context, L"SfsWins::HandleDestroy(): Invalid provider ID");
		else
		{
			auto_ptr<SfsWin> sfsWin(it->second);
			s_sfsWins.erase(it);
			sfsWin->HandleDestroy(context);
		}
	}

private:
	typedef map<unsigned int, SfsWin*> SfsWinMap;
	static SfsWinMap s_sfsWins;
};

SfsWins::SfsWinMap SfsWins::s_sfsWins;



int  __cdecl GetSfsVersion() { return SFS_VERSION; }

bool __cdecl HandleSfsInit(unsigned int id, SFS_Init const* init, SFS_Context* context)
{
	__try
	{
		return SfsWins::HandleInit(id, init, context);
	}
	__except (EXCEPTION_EXECUTE_HANDLER)
	{
		context->SetException(context, L"HandleSfsInit() handler failed with unhandled exception");
		return false; // Note: return code is irrelevant at this point.
	}
}

void __cdecl HandleSfsRequest(unsigned int id, SFS_Request const* request, SFS_Context* context)
{
	__try
	{
		SfsWins::HandleRequest(id, request, context);
	}
	__except (EXCEPTION_EXECUTE_HANDLER)
	{
		context->SetException(context, L"HandleSfsRequest() handler failed with unhandled exception");
	}
}

void __cdecl HandleSfsDestroy(unsigned int id, SFS_Context* context)
{
	__try
	{
		SfsWins::HandleDestroy(id, context);
	}
	__except (EXCEPTION_EXECUTE_HANDLER)
	{
		context->SetException(context, L"HandleSfsDestory() handler failed with unhandled exception");
	}
}
