#include "SfsWinUtilities.h"

// STL
#include <sstream>


using namespace std;



void AutoHandle::Set(HANDLE h)
{
	if (Valid())
		CloseHandle(m_h);
	m_h = h;
}


void AutoCryptContext::Reset()
{
	if (m_valid)
		CryptReleaseContext(m_prov, 0);
	m_valid = false;
}

void AutoCryptContext::Set(HCRYPTPROV prov)
{
	Reset();
	m_prov = prov;
	m_valid = true;
}

void AutoCryptHash::Reset() 
{ 
	if (m_valid)
		CryptDestroyHash(m_hash);
	m_valid = false;
}

void AutoCryptHash::Set(HCRYPTHASH hash)
{
	Reset();
	m_hash = hash;	
	m_valid = true;
}


char const* WException::what() const
{
	if (m_a.empty())
		if (!WideToNarrow(What(), m_a, CP_ACP))
			m_a = "[WException: conversion of exception description to ANSI code page failed]";
	return m_a.c_str();
}

wchar_t const* WinApiError::What() const 
{
	if (m_what.empty())
	{	
		std::wstringstream ss;
		ss << m_operation << L" failed with Windows error " << m_errorCode << L".";
		m_what = ss.str();
	}

	return m_what.c_str();
}


bool WideToNarrow(wchar_t const* wide, string& narrow, unsigned int codePage)
{
	narrow.clear();

	if (wide && wide[0])
	{
		int c = WideCharToMultiByte(codePage, 0, wide, -1, 0, 0, 0, 0);
		if (!c)
			return false;

		AutoArray<char> data(new char[c]);
		c = WideCharToMultiByte(codePage, 0, wide, -1, data.Get(), c, 0, 0);
		if (!c)
			return false;

		narrow = data.Get();
	}

	return true;
}

bool NarrowToWide(char const* narrow, unsigned int codePage, wstring& wide)
{
	wide.clear();

	if (narrow && narrow[0])
	{
		DWORD flags;
		if (codePage == CP_UTF8 || codePage == CP_UTF7)
			flags = 0;
		else
			flags = MB_PRECOMPOSED | MB_ERR_INVALID_CHARS;

		int c = MultiByteToWideChar(codePage, flags, narrow, -1, 0, 0);
		if (!c)
		{
			if (flags && GetLastError() == ERROR_INVALID_FLAGS)
				c = MultiByteToWideChar(codePage, flags = 0, narrow, -1, 0, 0);
			
			if (!c)	
				return false;
		}

		AutoArray<wchar_t> data(new wchar_t[c]);
		c = MultiByteToWideChar(codePage, flags, narrow, -1, data.Get(), c);
		if (!c)
			return false;

		wide = data.Get();
	}

	return true;
}


bool UxtimeToFtime(time_t uxt, FILETIME& ft)
{
	unsigned __int64 uxt64 = uxt;
	
	unsigned __int64 ft64 = uxt64;
	ft64 += 11644473600;
	ft64 *= 10000000;

	ft.dwHighDateTime = (DWORD) ((ft64 >> 32) & 0xFFFFFFFF);
	ft.dwLowDateTime = (DWORD) (ft64 & 0xFFFFFFFF);

	if (ft64 < uxt64)
		return false;
	else
		return true;
}

bool FtimeToUxtime(FILETIME ft, time_t& uxt)
{
	unsigned __int64 ft64 = ft.dwHighDateTime;
	ft64 <<= 32;
	ft64 |= ft.dwLowDateTime;

	unsigned __int64 uxt64 = ft64;
	uxt64 /= 10000000;
	uxt64 -= 11644473600;

	uxt = (time_t) uxt64;
	
	if (uxt64 > ft64 || uxt < 0 || uxt64 != (unsigned __int64) uxt)
		return false;
	else
		return true;
}
